local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- LibMapData
local LibMapData = LibStub:GetLibrary("LibMapData-1.0")

-- LibTourist
local LibTourist = LibStub:GetLibrary("LibTourist-3.0")

-- LibNodeTypeInfo
local LibNTI = LibStub:GetLibrary("LibNodeTypeInfo-1.0")

-- the module name
local MODULE      = "GatherMate2"
local DESCRIPTION = L["Module that uses GatherMate2 as data source."]

-- get GatherMate2
local GatherMate2 = _G["GatherMate2"]

-- local functions
local pairs = pairs

-- constants
local GM2_MINING    = "Mining"
local GM2_HERBALISM = "Herb Gathering"
local GM2_FISHING   = "Fishing"
local GM2_LOGGING   = "Logging"

-- lookup tables
local mapProfession = {
	gm2ToNTI = {
		[GM2_MINING]    = LibNTI.PROFESSION_MINING,
		[GM2_HERBALISM] = LibNTI.PROFESSION_HERBALISM,
		[GM2_FISHING]   = LibNTI.PROFESSION_FISHING,
		[GM2_LOGGING]   = LibNTI.PROFESSION_LOGGING,
	},
	ntiToGM2 = {
		[LibNTI.PROFESSION_MINING]    = GM2_MINING,
		[LibNTI.PROFESSION_HERBALISM] = GM2_HERBALISM,
		[LibNTI.PROFESSION_FISHING]   = GM2_FISHING,
		[LibNTI.PROFESSION_LOGGING]   = GM2_LOGGING,
	},
}

-- aux variables
local _

-- the module
local Inlet = NS:CreateInlet(MODULE, DESCRIPTION)

Inlet:SetReady(GatherMate2 and LibMapData and LibNTI and LibTourist and true or false)

-- variables
Inlet.nodeTypesGM2 = {}

-- register
NS:RegisterModule(Inlet)

function Inlet:GetNodesIterator(zoneId, profession)
	if not self:IsReady() then
		return
	end
	
	return self:IterateNodes(zoneId, mapProfession.ntiToGM2[profession])
end

do --Do-end block for iterator
	local emptyTbl = {}
	local tablestack = setmetatable({}, {__mode = 'k'})

	local function dbNodeIter(t, prestate)
		--Addon:Debug("dbNodeIter: " .. tostring(t) .. "/" .. tostring(prestate))
		if not t then 
			return nil 
		end

		local iterator = t.iterator
		local tbl      = t.t
		
		if iterator then
			local encLoc, nodeId = iterator(tbl, prestate)
			
			--	return encLoc, GatherMate2.reverseNodeIDs[t.profession] and GatherMate2.reverseNodeIDs[t.profession][nodeId]
			while encLoc do
				if Inlet.nodeTypesGM2[nodeId] then
					-- state, encLoc, nodeName
					return encLoc, encLoc, GatherMate2.reverseNodeIDs[t.profession] and GatherMate2.reverseNodeIDs[t.profession][nodeId]
				end
				
				encLoc, nodeId = iterator(tbl, encLoc)
			end	
		end
		
		tablestack[t] = true
		return nil, nil		
	end

	function Inlet:IterateNodes(zoneId, profession)
		local tbl = next(tablestack) or {}		

		tablestack[tbl] = nil
				
		local iterator, t, state = GatherMate2:GetNodesForZone(zoneId, profession, true)
		
		tbl.iterator = iterator
		tbl.t = t
		tbl.profession = profession
		
		return dbNodeIter, tbl, state
	end
end

-- implement update function which is called after node type selection changed in magneto
function Inlet:UpdateNodeSelection(event)
	if not self:IsReady() then
		return
	end
	
	NS:ClearTable(self.nodeTypesGM2)
	
	for nodeId, _ in Addon:IterateNodeTypeSelection() do
		local profession = LibNTI:GetProfession(nodeId)
		local nodeName   = LibNTI:GetName(profession, nodeId)
		
		local professionGM2 = mapProfession.ntiToGM2[profession]
		local nodeIdGM2     = GatherMate2.nodeIDs[professionGM2] and GatherMate2.nodeIDs[professionGM2][nodeName]
		
		if nodeIdGM2 then
			self.nodeTypesGM2[nodeIdGM2] = true
		end
	end
end

-- test
function Inlet:Debug(msg)
	Addon:Debug("(Inlet:" .. MODULE .. ") " .. msg)
end
