local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- the module name
local MODULE      = "BattlefieldMinimap"
local DESCRIPTION = L["Module that provides direction arrow on the Battlefield-Minimap."]

-- local functions
local cos     = math.cos
local sin     = math.sin

local GetCVar         = _G.GetCVar
local GetPlayerFacing = _G.GetPlayerFacing

-- local BattlefieldMinimap = BattlefieldMinimap

local PI               = math.pi
local RAD              = 180 / PI
local SQRT8            = math.sqrt(8)
local ONE_DIV_SQRT_8   = 1 / SQRT8

local DIST_FROM_CENTER = 25
local ARROW_SIZE = 12 

local ARROW = "Interface\\Addons\\"..ADDON.."\\minimap-magnet.tga"

-- aux variables
local _

-- the module
local Outlet = NS:CreateOutlet(MODULE, DESCRIPTION)

Outlet:SetReady(true)

-- register
NS:RegisterModule(Outlet)

-- implement functions for output
function Outlet:Refresh()
	if not self:IsOperational() or not BattlefieldMinimap then
		return
	end
	
	self:SetupArrow()
	
	local magnitude, direction, sinDir, cosDir = Addon:GetForce()

	if magnitude == 0 then
		self:Clear()
		
		return
	end
		
	local pX, pY = NS:GetPlayerPosition()
	local width, height = BattlefieldMinimap:GetWidth(), BattlefieldMinimap:GetHeight()
			
	local x = (pX * width) + (-(DIST_FROM_CENTER) * sinDir)
	local y = ((1 - pY) * height) + (DIST_FROM_CENTER * cosDir)
	
	-- border checks
	if x < ARROW_SIZE then
		x = ARROW_SIZE
	elseif x > width - ARROW_SIZE then
		x = width - ARROW_SIZE
	end

	if y < ARROW_SIZE then
		y = ARROW_SIZE
	elseif y > height - ARROW_SIZE then
		y = height - ARROW_SIZE
	end
	
	-- determine rotated and scaled texture coordinates
	local gX = (-sinDir + cosDir) * 0.5
	local gY = (-sinDir - cosDir) * 0.5
	
	self.arrow.texture:SetTexCoord(
		0.5 - gX, 0.5 + gY,
		0.5 + gY, 0.5 + gX,
		0.5 - gY, 0.5 - gX,
		0.5 + gX, 0.5 - gY)
				
	self.arrow:SetPoint("CENTER", BattlefieldMinimap, "BOTTOMLEFT", x, y)
	
	self.arrow:Show()
end

function Outlet:Clear()
	if not self:IsReady() or not BattlefieldMinimap then
		return
	end

	self:SetupArrow()
	
	if self.arrow:IsShown() then
		self.arrow:Hide()
	end
end

function Outlet:SetupArrow()
	if not self.arrow and BattlefieldMinimap then
		self.arrow = CreateFrame("Frame", ADDON .. "_" .. MODULE .. "_BFMinimap_Arrow", BattlefieldMinimap)
		self.arrow:SetHeight(ARROW_SIZE)
		self.arrow:SetWidth(ARROW_SIZE)
		self.arrow:SetParent(BattlefieldMinimap)
		self.arrow:SetFrameStrata(BattlefieldMinimap:GetFrameStrata())
		self.arrow:SetFrameLevel(BattlefieldMinimap:GetFrameLevel() + 7)

		local texture = self.arrow:CreateTexture(ADDON .. "_" .. MODULE .. "_BFMinimap_Arrow_Tex", "OVERLAY")
		self.arrow.texture = texture
		texture:SetAllPoints(self.arrow)
		texture:SetTexture(ARROW)
		texture:SetTexCoord(0, 1, 0, 1)
	end
end

-- test
function Outlet:Debug(msg)
	Addon:Debug("(Outlet:" .. MODULE .. ") " .. msg)
end
