local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- LibMapData
local LibMapData = LibStub:GetLibrary("LibMapData-1.0")

-- the module name
local MODULE      = "TomTom"
local DESCRIPTION = L["Module that provides TomTom direction arrow."]

-- locales
local TomTom           = TomTom
local TomTomCrazyArrow = TomTomCrazyArrow

local strfind   = strfind
local strformat = string.format
local abs       = math.abs
local floor     = math.floor
local log10     = math.log10

local GetPlayerFacing = _G.GetPlayerFacing

local PI         = math.pi
local TWOPI      = PI * 2
local ONE_DIV_PI = 1 / PI

local STAR = "*"

-- aux variables
local _

-- the module
local Outlet = NS:CreateOutlet(MODULE, DESCRIPTION)

Outlet:SetReady(TomTom and TomTomCrazyArrow and true or false)

Outlet.magnitude    = 0
Outlet.direction    = 0
Outlet.nodeCount    = 0
Outlet.visitedCount = 0
Outlet.percentage   = 0

Outlet.force = 0

Outlet.stars = 1
Outlet.angle = 0

Outlet.caHijacked = nil
Outlet.caLastText = ""
Outlet.caWasNorm  = true

-- register
NS:RegisterModule(Outlet)

--[[ TODO:
  - remove force display
  - display number of nodes in visiting distance?
]]

-- implement functions for output
function Outlet:Refresh()
	if not self:IsOperational() then
		return
	end
	
	local magnitude, direction, _, _, rating, totalCount, visitedCount = Addon:GetForce()
	
	if magnitude == 0 then
		self:Clear()
		return
	end
	
	self.magnitude    = magnitude
	self.direction    = direction	
	self.stars        = rating
	self.nodeCount    = totalCount
	self.visitedCount = visitedCount
	self.percentage   = totalCount > 0 and visitedCount*100 / totalCount or 0
	
	self:CrazyArrowHijack()
end

function Outlet:Clear()
	if not self:IsReady() then
		return
	end
	
	self:CrazyArrowRelease()
end

function Outlet:CrazyArrowPrepareHijack()
    self.caLastText = TomTomCrazyArrow.title:GetText()
	self.caWasNorm  = not (strfind(TomTomCrazyArrow.arrow:GetTexture(), "UP") and true or false)
	
    self:CrazyArrowShowNormal(true)
end

function Outlet:CrazyArrowRelease()
	if self.caHijacked then
		self:CrazyArrowShowNormal(self.caWasNorm)
		TomTomCrazyArrow.title:SetText(self.caLastText)
	
		TomTom:ReleaseCrazyArrow()
		
		self.caHijacked = nil
	end
end

function Outlet:CrazyArrowShowNormal(normal)
	if normal then
		TomTomCrazyArrow.arrow:SetHeight(56)
		TomTomCrazyArrow.arrow:SetWidth(42)
		TomTomCrazyArrow.arrow:SetTexture("Interface\\AddOns\\TomTom\\Images\\Arrow")
	else
		TomTomCrazyArrow.arrow:SetHeight(70)
		TomTomCrazyArrow.arrow:SetWidth(53)
		TomTomCrazyArrow.arrow:SetTexture("Interface\\AddOns\\TomTom\\Images\\Arrow-UP")
	end
end
		
function Outlet:CrazyArrowSetDirection(angle)
	if not angle then 
		angle = 0 
	end
	
	local cell = floor(angle / TWOPI * 108 + 0.5) % 108
	local column = cell % 9
	local row = floor(cell / 9)
	local xstart = (column * 56) / 512
	local ystart = (row * 42) / 512
	local xend = ((column + 1) * 56) / 512
	local yend = ((row + 1) * 42) / 512
	
	TomTomCrazyArrow.arrow:SetTexCoord(xstart, xend, ystart, yend)
	
	self:CrazyArrowSetColor(angle)
end

function Outlet:CrazyArrowSetColor(angle)
	local perc = abs(PI - abs(angle)) * ONE_DIV_PI
    local g = 1
    local r = 0
	
    if perc > 0.5 then
      g = 1
      r = perc * -2 + 2
    else
      r = 1
      g = perc * 2
    end
	
    TomTom:SetCrazyArrowColor(r, g, 0)
end

function Outlet:CrazyArrowHijack()
    if TomTom:CrazyArrowIsHijacked() then 
		return
	end

	self:CrazyArrowPrepareHijack()
	
	TomTom:HijackCrazyArrow(function(self, elapsed)
		local direction = Outlet.direction - GetPlayerFacing()
		
		Outlet:CrazyArrowSetDirection(direction)	
		
		local info = L["Visited"] .. ": " .. strformat(L["%.2f%% (%d of %d)"], Outlet.percentage, Outlet.visitedCount, Outlet.nodeCount)
		
		TomTom:SetCrazyArrowTitle(L["Magneto Force"], STAR:rep(Outlet.stars), info)
	end)
	
	self.caHijacked = true
end

-- test
function Outlet:Debug(msg)
	Addon:Debug("(Outlet:" .. MODULE .. ") " .. msg)
end
